@@ -8,6 +8,16 @@ import bisect
 import heapq
 import threading
 import lucene
+try:
+    from java.lang import Integer
+    from java.util import Arrays, HashSet
+    from org.apache.lucene import document, index, queries, search, util
+    from org.apache.lucene.search import highlight, spans, vectorhighlight
+    from org.apache.pylucene import search as search_
+    from org.apache.pylucene.queryparser.classic import PythonQueryParser
+except ImportError:
+    from lucene import Integer, Arrays, HashSet, PythonQueryParser
+    document = index = search = util = highlight = queries = spans = vectorhighlight = search_ = lucene
 
 class Query(object):
     """Inherited lucene Query, with dynamic base class acquisition.
@@ -19,31 +29,34 @@ class Query(object):
         base.__init__(self, *args)
     def filter(self, cache=True):
         "Return lucene CachingWrapperFilter, optionally just QueryWrapperFilter."
-        if isinstance(self, lucene.PrefixQuery):
-            filter = lucene.PrefixFilter(self.getPrefix())
-        elif isinstance(self, lucene.TermRangeQuery):
-            filter = lucene.TermRangeFilter(self.field, self.lowerTerm, self.upperTerm, self.includesLower(), self.includesUpper())
-        elif isinstance(self, lucene.TermQuery):
-            filter = lucene.TermsFilter()
-            filter.addTerm(self.getTerm())
+        if isinstance(self, search.PrefixQuery):
+            filter = search.PrefixFilter(self.getPrefix())
+        elif isinstance(self, search.TermRangeQuery):
+            filter = search.TermRangeFilter(self.field, self.lowerTerm, self.upperTerm, self.includesLower(), self.includesUpper())
+        elif isinstance(self, search.TermQuery):
+            if hasattr(queries.TermsFilter, 'addTerm'):
+                filter = queries.TermsFilter()
+                filter.addTerm(self.getTerm())
+            else:
+                filter = queries.TermsFilter([self.getTerm()])
         else:
-            filter = lucene.QueryWrapperFilter(self)
-        return lucene.CachingWrapperFilter(filter) if cache else filter
+            filter = search.QueryWrapperFilter(self)
+        return search.CachingWrapperFilter(filter) if cache else filter
     def terms(self):
         "Generate set of query term items."
-        terms = lucene.HashSet().of_(lucene.Term)
+        terms = HashSet().of_(index.Term)
         self.extractTerms(terms)
         for term in terms:
             yield term.field(), term.text()
     @classmethod
     def term(cls, name, value, boost=1.0):
         "Return lucene TermQuery."
-        self = cls(lucene.TermQuery, lucene.Term(name, value))
+        self = cls(search.TermQuery, index.Term(name, value))
         self.boost = boost
         return self
     @classmethod
     def boolean(cls, occur, *queries, **terms):
-        self = BooleanQuery(lucene.BooleanQuery)
+        self = BooleanQuery(search.BooleanQuery)
         for query in queries:
             self.add(query, occur)
         for name, values in terms.items():
@@ -53,15 +66,15 @@ class Query(object):
     @classmethod
     def any(cls, *queries, **terms):
         "Return `BooleanQuery`_ (OR) from queries and terms."
-        return cls.boolean(lucene.BooleanClause.Occur.SHOULD, *queries, **terms)
+        return cls.boolean(search.BooleanClause.Occur.SHOULD, *queries, **terms)
     @classmethod
     def all(cls, *queries, **terms):
         "Return `BooleanQuery`_ (AND) from queries and terms."
-        return cls.boolean(lucene.BooleanClause.Occur.MUST, *queries, **terms)
+        return cls.boolean(search.BooleanClause.Occur.MUST, *queries, **terms)
     @classmethod
     def disjunct(cls, multiplier, *queries, **terms):
         "Return lucene DisjunctionMaxQuery from queries and terms."
-        self = cls(lucene.DisjunctionMaxQuery, lucene.Arrays.asList(queries), multiplier)
+        self = cls(search.DisjunctionMaxQuery, Arrays.asList(queries), multiplier)
         for name, values in terms.items():
             for value in ([values] if isinstance(values, basestring) else values):
                 self.add(cls.term(name, value))
@@ -70,8 +83,8 @@ class Query(object):
     def span(cls, *term):
         "Return `SpanQuery`_ from term name and value or a MultiTermQuery."
         if len(term) <= 1:
-            return SpanQuery(lucene.SpanMultiTermQueryWrapper, *term)
-        return SpanQuery(lucene.SpanTermQuery, lucene.Term(*term))
+            return SpanQuery(spans.SpanMultiTermQueryWrapper, *term)
+        return SpanQuery(spans.SpanTermQuery, index.Term(*term))
     @classmethod
     def near(cls, name, *values, **kwargs):
         """Return :meth:`SpanNearQuery <SpanQuery.near>` from terms.
@@ -81,41 +94,47 @@ class Query(object):
     @classmethod
     def prefix(cls, name, value):
         "Return lucene PrefixQuery."
-        return cls(lucene.PrefixQuery, lucene.Term(name, value))
+        return cls(search.PrefixQuery, index.Term(name, value))
     @classmethod
     def range(cls, name, start, stop, lower=True, upper=False):
         "Return lucene RangeQuery, by default with a half-open interval."
-        return cls(lucene.TermRangeQuery, name, start, stop, lower, upper)
+        try:
+            return cls(search.TermRangeQuery, name, start, stop, lower, upper)
+        except lucene.InvalidArgsError:
+            start, stop = (value if value is None else util.BytesRef(value) for value in (start, stop))
+        return cls(search.TermRangeQuery, name, start, stop, lower, upper)
     @classmethod
     def phrase(cls, name, *values):
         "Return lucene PhraseQuery.  None may be used as a placeholder."
-        self = cls(lucene.PhraseQuery)
-        for index, value in enumerate(values):
+        self = cls(search.PhraseQuery)
+        for idx, value in enumerate(values):
             if value is not None:
-                self.add(lucene.Term(name, value), index)
+                self.add(index.Term(name, value), idx)
         return self
     @classmethod
     def multiphrase(cls, name, *values):
         "Return lucene MultiPhraseQuery.  None may be used as a placeholder."
-        self = cls(lucene.MultiPhraseQuery)
-        for index, words in enumerate(values):
+        self = cls(search.MultiPhraseQuery)
+        for idx, words in enumerate(values):
             if isinstance(words, basestring):
                 words = [words]
             if words is not None:
-                self.add([lucene.Term(name, word) for word in words], index)
+                self.add([index.Term(name, word) for word in words], idx)
         return self
     @classmethod
     def wildcard(cls, name, value):
         "Return lucene WildcardQuery."
-        return cls(lucene.WildcardQuery, lucene.Term(name, value))
+        return cls(search.WildcardQuery, index.Term(name, value))
     @classmethod
-    def fuzzy(cls, name, value, minimumSimilarity=0.5, prefixLength=0):
+    def fuzzy(cls, name, value, minimumSimilarity=None, prefixLength=0):
         "Return lucene FuzzyQuery."
-        return cls(lucene.FuzzyQuery, lucene.Term(name, value), minimumSimilarity, prefixLength)
+        if minimumSimilarity is None:
+            minimumSimilarity = getattr(search.FuzzyQuery, 'defaultMaxEdits', 0.5)
+        return cls(search.FuzzyQuery, index.Term(name, value), minimumSimilarity, prefixLength)
     def __pos__(self):
         return Query.all(self)
     def __neg__(self):
-        return Query.boolean(lucene.BooleanClause.Occur.MUST_NOT, self)
+        return Query.boolean(search.BooleanClause.Occur.MUST_NOT, self)
     def __and__(self, other):
         return Query.all(self, other)
     def __rand__(self, other):
@@ -138,30 +157,32 @@ class BooleanQuery(Query):
     def __getitem__(self, index):
         return self.getClauses()[index]
     def __iand__(self, other):
-        self.add(other, lucene.BooleanClause.Occur.MUST)
+        self.add(other, search.BooleanClause.Occur.MUST)
         return self
     def __ior__(self, other):
-        self.add(other, lucene.BooleanClause.Occur.SHOULD)
+        self.add(other, search.BooleanClause.Occur.SHOULD)
         return self
     def __isub__(self, other):
-        self.add(other, lucene.BooleanClause.Occur.MUST_NOT)
+        self.add(other, search.BooleanClause.Occur.MUST_NOT)
         return self
 
 class SpanQuery(Query):
     "Inherited lucene SpanQuery with additional span constructors."
     def filter(self, cache=True):
         "Return lucene CachingSpanFilter, optionally just SpanQueryFilter."
-        filter = lucene.SpanQueryFilter(self)
-        return lucene.CachingSpanFilter(filter) if cache else filter
+        if not hasattr(search, 'SpanQueryFilter'):
+            return Query.filter(self, cache)
+        filter = search.SpanQueryFilter(self)
+        return search.CachingSpanFilter(filter) if cache else filter
     def __getitem__(self, slc):
-        start, stop, step = slc.indices(lucene.Integer.MAX_VALUE)
+        start, stop, step = slc.indices(Integer.MAX_VALUE)
         assert step == 1, 'slice step is not supported'
-        return SpanQuery(lucene.SpanPositionRangeQuery, self, start, stop)
+        return SpanQuery(spans.SpanPositionRangeQuery, self, start, stop)
     def __sub__(self, other):
-        return SpanQuery(lucene.SpanNotQuery, self, other)
-    def __or__(*spans):
-        return SpanQuery(lucene.SpanOrQuery, spans)
-    def near(*spans, **kwargs):
+        return SpanQuery(spans.SpanNotQuery, self, other)
+    def __or__(*spans_):
+        return SpanQuery(spans.SpanOrQuery, spans_)
+    def near(*spans_, **kwargs):
         """Return lucene SpanNearQuery from SpanQueries.
         
         :param slop: default 0
@@ -169,47 +190,39 @@ class SpanQuery(Query):
         :param collectPayloads: default True
         """
         args = map(kwargs.get, ('slop', 'inOrder', 'collectPayloads'), (0, True, True))
-        return SpanQuery(lucene.SpanNearQuery, spans, *args)
+        return SpanQuery(spans.SpanNearQuery, spans_, *args)
     def mask(self, name):
         "Return lucene FieldMaskingSpanQuery, which allows combining span queries from different fields."
-        return SpanQuery(lucene.FieldMaskingSpanQuery, self, name)
+        return SpanQuery(spans.FieldMaskingSpanQuery, self, name)
     def payload(self, *values):
         "Return lucene SpanPayloadCheckQuery from payload values."
-        base = lucene.SpanNearPayloadCheckQuery if lucene.SpanNearQuery.instance_(self) else lucene.SpanPayloadCheckQuery
-        return SpanQuery(base, self, lucene.Arrays.asList(list(map(lucene.JArray_byte, values))))
+        base = spans.SpanNearPayloadCheckQuery if spans.SpanNearQuery.instance_(self) else spans.SpanPayloadCheckQuery
+        return SpanQuery(base, self, Arrays.asList(list(map(lucene.JArray_byte, values))))
 
-class Collector(lucene.PythonCollector):
-    "Collect all ids and scores efficiently."
-    def __init__(self):
-        lucene.PythonCollector.__init__(self)
-        self.scores = {}
-    def collect(self, id, score):
-        self.scores[id + self.base] = score
-    def setNextReader(self, reader, base):
-        self.base = base
-    def acceptsDocsOutOfOrder(self):
-        return True
-    def sorted(self, key=None, reverse=False):
-        "Return ordered ids and scores."
-        ids = sorted(self.scores)
-        if key is None:
-            key, reverse = self.scores.__getitem__, True
-        ids.sort(key=key, reverse=reverse)
-        return ids, list(map(self.scores.__getitem__, ids))
+class BooleanFilter(queries.BooleanFilter):
+    "Inherited lucene BooleanFilter similar to BooleanQuery."
+    def __init__(self, occur, *filters):
+        queries.BooleanFilter.__init__(self)
+        for filter in filters:
+            self.add(queries.FilterClause(filter, occur))
+    @classmethod
+    def all(cls, *filters):
+        "Return `BooleanFilter`_ (AND) from filters."
+        return cls(search.BooleanClause.Occur.MUST, *filters)
 
-class TermsFilter(lucene.CachingWrapperFilter):
+class TermsFilter(search.CachingWrapperFilter):
     """Caching filter based on a unique field and set of matching values.
     Optimized for many terms and docs, with support for incremental updates.
     Suitable for searching external metadata associated with indexed identifiers.
-    Call :meth:`refresh` to cache a new (or reopened) reader.
+    Call :meth:`refresh` to cache a new (or reopened) searcher.
     
     :param field: field name
     :param values: initial term values, synchronized with the cached filters
     """
     ops = {'or': 'update', 'and': 'intersection_update', 'andNot': 'difference_update'}
     def __init__(self, field, values=()):
-        assert hasattr(lucene, 'FixedBitSet'), 'requires FixedBitSet introduced in lucene 3.4'
-        lucene.CachingWrapperFilter.__init__(self, lucene.TermsFilter())
+        assert lucene.VERSION >= '3.5', 'requires FixedBitSet set operations introduced in lucene 3.5'
+        search.CachingWrapperFilter.__init__(self, search.QueryWrapperFilter(search.MatchAllDocsQuery()))
         self.field = field
         self.values = set(values)
         self.readers = set()
@@ -217,20 +230,29 @@ class TermsFilter(lucene.CachingWrapperFilter):
     def filter(self, values, cache=True):
         "Return lucene TermsFilter, optionally using the FieldCache."
         if cache:
-            return lucene.FieldCacheTermsFilter(self.field, tuple(values))
-        filter, term = lucene.TermsFilter(), lucene.Term(self.field)
-        for value in values:
-            filter.addTerm(term.createTerm(value))
+            return search.FieldCacheTermsFilter(self.field, tuple(values))
+        terms = [index.Term(self.field, value) for value in values]
+        if not hasattr(queries.TermsFilter, 'addTerm'):
+            return queries.TermsFilter(terms)
+        filter = queries.TermsFilter()
+        for term in terms:
+            filter.addTerm(term)
         return filter
     def apply(self, filter, op, readers):
         for reader in readers:
-            bitset = lucene.FixedBitSet.cast_(self.getDocIdSet(reader))
-            getattr(bitset, op)(filter.getDocIdSet(reader).iterator())
+            try:
+                args = [reader.context, None] if hasattr(index.IndexReader, 'context') else [reader]
+                bitset = util.FixedBitSet.cast_(self.getDocIdSet(*args))
+                if reader not in self.readers:
+                    bitset.clear(0, bitset.length())
+                getattr(bitset, op)(filter.getDocIdSet(*args).iterator())
+            except lucene.JavaError as exc:
+                assert not reader.refCount, exc
     def update(self, values, op='or', cache=True):
         """Update allowed values and corresponding cached bitsets.
         
         :param values: additional term values
-        :param op: set operation used to combine terms and docs
+        :param op: set operation used to combine terms and docs: *and*, *or*, *andNot*
         :param cache: optionally cache all term values using FieldCache
         """
         values = tuple(values)
@@ -238,9 +260,9 @@ class TermsFilter(lucene.CachingWrapperFilter):
         with self.lock:
             getattr(self.values, self.ops[op])(values)
             self.apply(filter, op, self.readers)
-    def refresh(self, reader):
-        "Refresh cached bitsets of current values for new segments of top-level reader."
-        readers = set(reader.sequentialSubReaders)
+    def refresh(self, searcher):
+        "Refresh cached bitsets of current values for new segments of searcher."
+        readers = set(searcher.readers)
         with self.lock:
             self.apply(self.filter(self.values), 'or', readers - self.readers)
             self.readers = set(reader for reader in readers | self.readers if reader.refCount)
@@ -251,7 +273,28 @@ class TermsFilter(lucene.CachingWrapperFilter):
         "Discard a few term values."
         self.update(values, op='andNot', cache=False)
 
-class SortField(lucene.SortField):
+class Comparator(object):
+    "Chained arrays with bisection lookup."
+    def __init__(self, items):
+        self.arrays, self.offsets = [], [0]
+        for array, size in items:
+            self.arrays.append(array)
+            self.offsets.append(len(self) + size)
+    def __len__(self):
+        return self.offsets[-1]
+    def __getitem__(self, id):
+        idx = bisect.bisect_right(self.offsets, id) - 1
+        id -= self.offsets[idx]
+        array = self.arrays[idx]
+        if lucene.VERSION < '4':
+            return array[id]
+        if not isinstance(array, index.BinaryDocValues):
+            return array.get(id)
+        br = util.BytesRef()
+        array.get(id, br)
+        return br.utf8ToString()
+
+class SortField(search.SortField):
     """Inherited lucene SortField used for caching FieldCache parsers.
     
     :param name: field name
@@ -262,44 +305,49 @@ class SortField(lucene.SortField):
     def __init__(self, name, type='string', parser=None, reverse=False):
         type = self.typename = getattr(type, '__name__', type).capitalize()
         if parser is None:
-            parser = getattr(lucene.SortField, type.upper())
-        elif not lucene.FieldCache.Parser.instance_(parser):
-            base = getattr(lucene, 'Python{0}Parser'.format(type))
-            namespace = {'parse' + type: staticmethod(parser)}
+            parser = getattr(getattr(self, 'Type', self), type.upper())
+        elif not search.FieldCache.Parser.instance_(parser):
+            base = getattr(search_, 'Python{0}Parser'.format(type))
+            namespace = {'parse' + type: staticmethod(parser), 'termsEnum': lambda self, terms: terms.iterator(None)}
             parser = object.__class__(base.__name__, (base,), namespace)()
-        lucene.SortField.__init__(self, name, parser, reverse)
+        search.SortField.__init__(self, name, parser, reverse)
     def array(self, reader):
-        method = getattr(lucene.FieldCache.DEFAULT, 'get{0}s'.format(self.typename))
-        return method(reader, self.field, *[self.parser][:bool(self.parser)])
-    def comparator(self, reader):
-        "Return indexed values from default FieldCache using the given top-level reader."
-        readers = reader.sequentialSubReaders
-        if lucene.MultiReader.instance_(reader):
-            readers = itertools.chain.from_iterable(reader.sequentialSubReaders for reader in readers)
-        arrays = list(map(self.array, readers))
-        if len(arrays) <= 1:
-            return arrays[0]
-        cls, = set(map(type, arrays))
-        index, result = 0, cls(sum(map(len, arrays)))
-        for array in arrays:
-            lucene.System.arraycopy(array, 0, result, index, len(array))
-            index += len(array)
-        return result
+        if lucene.VERSION >= '4' and self.typename == 'String':
+            return search.FieldCache.DEFAULT.getTerms(reader, self.field)
+        args = []
+        method = getattr(search.FieldCache.DEFAULT, 'get{0}s'.format(self.typename))
+        if self.parser:
+            args.append(self.parser)
+        if lucene.VERSION >= '4':
+            args.append(False)
+        return method(reader, self.field, *args)
+    def comparator(self, searcher):
+        "Return indexed values from default FieldCache using the given searcher."
+        return Comparator((self.array(reader), reader.maxDoc()) for reader in searcher.readers)
     def filter(self, start, stop, lower=True, upper=False):
         "Return lucene FieldCacheRangeFilter based on field and type."
-        method = getattr(lucene.FieldCacheRangeFilter, 'new{0}Range'.format(self.typename))
+        method = getattr(search.FieldCacheRangeFilter, 'new{0}Range'.format(self.typename))
         return method(self.field, self.parser, start, stop, lower, upper)
     def terms(self, filter, *readers):
         "Generate field cache terms from docs which match filter from all segments."
         for reader in readers:
-            array, it = self.array(reader), filter.getDocIdSet(reader).iterator()
-            try:
-                while True:
-                    yield array[it.nextDoc()]
-            except IndexError:
-                pass
+            args = [reader.context, reader.liveDocs] if hasattr(reader, 'liveDocs') else [reader]
+            array, docset = self.array(reader), filter.getDocIdSet(*args)
+            ids = iter(docset.iterator().nextDoc, search.DocIdSetIterator.NO_MORE_DOCS)
+            if lucene.VERSION < '4':
+                for id in ids:
+                    yield array[id]
+            else:
+                br = util.BytesRef()
+                if isinstance(array, index.BinaryDocValues):
+                    for id in ids:
+                        array.get(id, br)
+                        yield br.utf8ToString()
+                else:
+                    for id in ids:
+                        yield array.get(id)
 
-class Highlighter(lucene.Highlighter):
+class Highlighter(highlight.Highlighter):
     """Inherited lucene Highlighter with stored analysis options.
     
     :param searcher: `IndexSearcher`_ used for analysis, scoring, and optionally text retrieval
@@ -313,11 +361,11 @@ class Highlighter(lucene.Highlighter):
     """
     def __init__(self, searcher, query, field, terms=False, fields=False, tag='', formatter=None, encoder=None):
         if tag:
-            formatter = lucene.SimpleHTMLFormatter('<{0}>'.format(tag), '</{0}>'.format(tag))
-        scorer = (lucene.QueryTermScorer if terms else lucene.QueryScorer)(query, *(searcher.indexReader, field) * (not fields))
-        lucene.Highlighter.__init__(self, *filter(None, [formatter, encoder, scorer]))
+            formatter = highlight.SimpleHTMLFormatter('<{0}>'.format(tag), '</{0}>'.format(tag))
+        scorer = (highlight.QueryTermScorer if terms else highlight.QueryScorer)(query, *(searcher.indexReader, field) * (not fields))
+        highlight.Highlighter.__init__(self, *filter(None, [formatter, encoder, scorer]))
         self.searcher, self.field = searcher, field
-        self.selector = lucene.MapFieldSelector([field])
+        self.selector = getattr(document, 'MapFieldSelector', HashSet)(Arrays.asList([field]))
     def fragments(self, doc, count=1):
         """Return highlighted text fragments.
         
@@ -325,10 +373,10 @@ class Highlighter(lucene.Highlighter):
         :param count: maximum number of fragments
         """
         if not isinstance(doc, basestring):
-            doc = self.searcher.doc(doc, self.selector)[self.field]
+            doc = getattr(search.IndexSearcher, 'document', search.IndexSearcher.doc)(self.searcher, doc, self.selector)[self.field]
         return doc and list(self.getBestFragments(self.searcher.analyzer, self.field, doc, count))
 
-class FastVectorHighlighter(lucene.FastVectorHighlighter):
+class FastVectorHighlighter(vectorhighlight.FastVectorHighlighter):
     """Inherited lucene FastVectorHighlighter with stored query.
     Fields must be stored and have term vectors with offsets and positions.
     
@@ -343,9 +391,9 @@ class FastVectorHighlighter(lucene.FastVectorHighlighter):
     """
     def __init__(self, searcher, query, field, terms=False, fields=False, tag='', fragListBuilder=None, fragmentsBuilder=None):
         if tag:
-            fragmentsBuilder = lucene.SimpleFragmentsBuilder(['<{0}>'.format(tag)], ['</{0}>'.format(tag)])
-        args = fragListBuilder or lucene.SimpleFragListBuilder(), fragmentsBuilder or lucene.SimpleFragmentsBuilder()
-        lucene.FastVectorHighlighter.__init__(self, not terms, not fields, *args)
+            fragmentsBuilder = vectorhighlight.SimpleFragmentsBuilder(['<{0}>'.format(tag)], ['</{0}>'.format(tag)])
+        args = fragListBuilder or vectorhighlight.SimpleFragListBuilder(), fragmentsBuilder or vectorhighlight.SimpleFragmentsBuilder()
+        vectorhighlight.FastVectorHighlighter.__init__(self, not terms, not fields, *args)
         self.searcher, self.field = searcher, field
         self.query = self.getFieldQuery(query)
     def fragments(self, id, count=1, size=100):
@@ -399,7 +447,7 @@ class SpellChecker(dict):
             groups = map(self.edits, edits, edits.values())
             edits = dict((edit, group[edit]) for group in groups for edit in group if edit not in previous)
 
-class SpellParser(lucene.PythonQueryParser):
+class SpellParser(PythonQueryParser):
     """Inherited lucene QueryParser which corrects spelling.
     Assign a searcher attribute or override :meth:`correct` implementation.
     """
@@ -407,15 +455,15 @@ class SpellParser(lucene.PythonQueryParser):
         "Return term with text replaced as necessary."
         field = term.field()
         for text in self.searcher.correct(field, term.text()):
-            return lucene.Term(field, text)
+            return index.Term(field, text)
         return term
     def rewrite(self, query):
         "Return term or phrase query with corrected terms substituted."
-        if lucene.TermQuery.instance_(query):
-            term = lucene.TermQuery.cast_(query).term
-            return lucene.TermQuery(self.correct(term))
-        query = lucene.PhraseQuery.cast_(query)
-        phrase = lucene.PhraseQuery()
+        if search.TermQuery.instance_(query):
+            term = search.TermQuery.cast_(query).term
+            return search.TermQuery(self.correct(term))
+        query = search.PhraseQuery.cast_(query)
+        phrase = search.PhraseQuery()
         for position, term in zip(query.positions, query.terms):
             phrase.add(self.correct(term), position)
         return phrase
