@@ -39,14 +39,14 @@ CherryPy and Lucene VM integration issues:
  * Recommended that the VM ignores keyboard interrupts (-Xrs) for clean server shutdown.
 """
 
-from future_builtins import filter, map
+from future_builtins import map
 import re
 import time
 import uuid
 import socket, httplib
 import heapq
 import collections
-import itertools, operator
+import itertools
 import os, optparse
 from email.utils import formatdate
 import contextlib
@@ -54,13 +54,21 @@ try:
     import simplejson as json
 except ImportError:
     import json
+import warnings
 import lucene
+try:
+    from org.apache.lucene import search
+except ImportError:
+    search = lucene
 import cherrypy
 try:
     from . import engine, client
 except ValueError:
     import engine, client
 
+if cherrypy.__version__ < '3.2':
+    warnings.warn('Support for cherrypy 3.1 will be removed in the next release.', DeprecationWarning)
+
 def tool(hook):
     "Return decorator to register tool at given hook point."
     def decorator(func):
@@ -123,7 +131,7 @@ def timer():
     response.headers['x-response-time'] = time.time() - response.time
 
 @tool('on_start_resource')
-def validate(etag=True, last_modified=True, max_age=None, expires=None):
+def validate(etag=True, last_modified=False, max_age=None, expires=None):
     """Return and validate caching headers.
     
     :param etag: return weak entity tag header based on index version and validate if-match headers
@@ -156,12 +164,6 @@ def params(**types):
 def multi(value):
     return value and value.split(',')
 
-def mapping(value):
-    if isinstance(value, dict):
-        return value
-    cherrypy.response.headers['warning'] = '199 lupyne "use an object instead of an array"'
-    return dict.fromkeys(value, True)
-
 class params:
     "Parameter parsing."
     @staticmethod
@@ -257,7 +259,7 @@ class WebSearcher(object):
     def sync(self, host, path=''):
         "Sync with remote index."
         path = '/' + '{0}/update/{1}/'.format(path, uuid.uuid1()).lstrip('/')
-        directory = lucene.FSDirectory.cast_(self.searcher.directory).directory.path
+        directory = self.searcher.path
         resource = client.Resource(host)
         names = sorted(set(resource.put(path)).difference(os.listdir(directory)))
         try:
@@ -283,11 +285,12 @@ class WebSearcher(object):
         if cherrypy.request.method == 'POST':
             self.sync(host, path)
             cherrypy.response.status = httplib.ACCEPTED
-        reader = self.searcher.indexReader
-        readers = reader.sequentialSubReaders if lucene.MultiReader.instance_(reader) else [reader]
-        return dict((unicode(reader.directory()), reader.numDocs()) for reader in readers)
+        if isinstance(self.searcher, engine.MultiSearcher):
+            readers = map(engine.indexers.IndexReader, self.searcher.sequentialSubReaders)
+            return dict((reader.directory.toString(), reader.numDocs()) for reader in readers)
+        return {self.searcher.directory.toString(): len(self.searcher)}
     @cherrypy.expose
-    @cherrypy.tools.json_in(process_body=mapping)
+    @cherrypy.tools.json_in(process_body=dict)
     @cherrypy.tools.allow(methods=['POST'])
     def update(self, **caches):
         """Refresh index version.
@@ -409,28 +412,22 @@ class WebSearcher(object):
             
             :return:
                 | {
-                | "query": *string*,
+                | "query": *string*\|null,
                 | "count": *int*\|null,
                 | "maxscore": *number*\|null,
-                | "docs": [{"__id__": *int*, "__score__": *number*, "__highlights__": {*string*: *array*,... }, *string*: *object*,... },... ],
+                | "docs": [{"__id__": *int*, "__score__": *number*, "__keys__": *array*,
+                    "__highlights__": {*string*: *array*,... }, *string*: *value*,... },... ],
                 | "facets": {*string*: {*string*: *int*,... },... },
                 | "groups": [{"count": *int*, "value": *value*, "docs": [*object*,... ]},... ]
                 | "spellcheck": {*string*: {*string*: [*string*,... ],... },... },
                 | }
         """
         searcher = self.searcher
-        reverse = False
         if sort is not None:
             sort = (re.match('(-?)(\w+):?(\w*)', field).groups() for field in sort)
             sort = [(name, (type or 'string'), (reverse == '-')) for reverse, name, type in sort]
-            if count is None:
-                with HTTPError(httplib.BAD_REQUEST, ValueError, AttributeError):
-                    reverse, = set(reverse for name, type, reverse in sort) # only one sort direction allowed with unlimited count
-                    comparators = [searcher.comparator(name, type) for name, type, reverse in sort]
-                sort = comparators[0].__getitem__ if len(comparators) == 1 else lambda id: tuple(map(operator.itemgetter(id), comparators))
-            else:
-                with HTTPError(httplib.BAD_REQUEST, AttributeError):
-                    sort = [searcher.sorter(name, type, reverse=reverse) for name, type, reverse in sort]
+            with HTTPError(httplib.BAD_REQUEST, AttributeError):
+                sort = [searcher.sorter(name, type, reverse=reverse) for name, type, reverse in sort]
         q = params.q(searcher, q, **options)
         qfilter = options.pop('filter', None)
         if qfilter is not None and qfilter not in searcher.filters:
@@ -438,18 +435,18 @@ class WebSearcher(object):
         qfilter = searcher.filters.get(qfilter)
         if mlt is not None:
             if q is not None:
-                mlt = searcher.search(q, count=mlt+1, sort=sort, reverse=reverse).ids[mlt]
+                mlt, = searcher.search(q, count=mlt+1, sort=sort)[mlt:].ids
             mltfields = options.pop('mlt.fields', ())
             with HTTPError(httplib.BAD_REQUEST, ValueError):
                 attrs = dict((key.partition('.')[-1], json.loads(options[key])) for key in options if key.startswith('mlt.'))
-            q = searcher.morelikethis(mlt, *mltfields, **attrs)
+            q = searcher.morelikethis(mlt, *mltfields, analyzer=searcher.analyzer, **attrs)
         if count is not None:
             count += start
         if count == 0:
             start = count = 1
         scores = options.get('sort.scores')
         scores = {'scores': scores is not None, 'maxscore': scores == 'max'}
-        hits = searcher.search(q, filter=qfilter, count=count, sort=sort, reverse=reverse, timeout=timeout, **scores)[start:]
+        hits = searcher.search(q, filter=qfilter, count=count, sort=sort, timeout=timeout, **scores)[start:]
         result = {'query': q and unicode(q), 'count': hits.count, 'maxscore': hits.maxscore}
         tag, enable = options.get('hl.tag', 'strong'), options.get('hl.enable', '')
         hlcount = options.get('hl.count', 1)
@@ -459,7 +456,7 @@ class WebSearcher(object):
         if fields is None:
             fields = {}
         else:
-            hits.fields = lucene.MapFieldSelector(list(itertools.chain(fields, multi)))
+            hits.select(*itertools.chain(fields, multi))
         with HTTPError(httplib.BAD_REQUEST, AttributeError):
             groups = hits.groupby(searcher.comparator(*group.split(':')).__getitem__) if group else [hits]
         result['groups'], limit = [], options.get('group.limit', len(groups))
@@ -477,7 +474,7 @@ class WebSearcher(object):
             result['groups'].append({'docs': [], 'count': len(hits), 'value': hits.value})
         if not group:
             result['docs'] = result.pop('groups')[0]['docs']
-        q = q or lucene.MatchAllDocsQuery()
+        q = q or search.MatchAllDocsQuery()
         if facets:
             facets = (tuple(facet.split(':')) if ':' in facet else facet for facet in facets)
             facets = result['facets'] = searcher.facets(q, *facets)
@@ -609,7 +606,7 @@ class WebIndexer(WebSearcher):
             self.refresh()
         return {unicode(self.indexer.directory): len(self.indexer)}
     @cherrypy.expose
-    @cherrypy.tools.json_in(process_body=mapping)
+    @cherrypy.tools.json_in(process_body=dict)
     @cherrypy.tools.allow(paths=[('POST',), ('GET', 'PUT', 'DELETE'), ('GET',)])
     def update(self, id='', name='', **options):
         """Commit index changes and refresh index version.
@@ -649,7 +646,7 @@ class WebIndexer(WebSearcher):
         if not name:
             return list(commit.fileNames)
         with HTTPError(httplib.NOT_FOUND, TypeError, AssertionError):
-            directory = lucene.FSDirectory.cast_(self.indexer.directory).directory.path
+            directory = self.searcher.path
             assert name in commit.fileNames, 'file not referenced in commit'
         return cherrypy.lib.static.serve_download(os.path.join(directory, name))
     @cherrypy.expose
