@@ -3,20 +3,32 @@ import unittest
 import os
 import tempfile, shutil
 import itertools
-import collections
 import warnings
 import datetime
 import math
 import bisect
 import contextlib
 import lucene
+try:
+    from java.io import StringReader
+    from org.apache.lucene import analysis, document, search, store, util
+    from org.apache.lucene.analysis import miscellaneous, standard
+    from org.apache.lucene.search import grouping, highlight, vectorhighlight
+    from org.apache.pylucene.search import PythonFilter
+except ImportError:
+    from lucene import StringReader, PythonFilter
+    analysis = document = search = store = util = miscellaneous = standard = grouping = highlight = vectorhighlight = lucene
 from lupyne import engine
 from . import fixture
+if not hasattr(analysis, 'PorterStemFilter'):
+    analysis.PorterStemFilter = analysis.en.PorterStemFilter
+if hasattr(analysis, 'core'):
+    analysis.WhitespaceAnalyzer, analysis.WhitespaceTokenizer = analysis.core.WhitespaceAnalyzer, analysis.core.WhitespaceTokenizer
 
 class typeAsPayload(engine.TokenFilter):
     "Custom implementation of lucene TypeAsPayloadTokenFilter."
     def setattrs(self):
-        self.payload = self.type.encode('utf8')
+        self.payload = self.type
 
 @contextlib.contextmanager
 def assertWarns(*categories):
@@ -25,9 +37,9 @@ def assertWarns(*categories):
     for message, category in itertools.izip_longest(messages, categories):
          assert issubclass(message.category, category), message
 
-class Filter(lucene.PythonFilter):
+class Filter(PythonFilter):
     "Broken filter to test errors are raised."
-    def getDocIdSet(self, indexReader):
+    def getDocIdSet(self, *args):
         assert False
 
 class BaseTest(unittest.TestCase):
@@ -41,11 +53,11 @@ class TestCase(BaseTest):
     def testInterface(self):
         "Indexer and document interfaces."
         self.assertRaises(TypeError, engine.IndexSearcher)
-        analyzer = lucene.StandardAnalyzer(lucene.Version.values()[-1])
-        stemmer = engine.Analyzer(analyzer, lucene.PorterStemFilter, typeAsPayload)
+        analyzer = lambda reader: standard.StandardTokenizer(util.Version.values()[-1], reader)
+        stemmer = engine.Analyzer(analyzer, analysis.PorterStemFilter, typeAsPayload)
         for token in stemmer.tokens('hello'):
             assert token.positionIncrement == 1
-            assert engine.TokenFilter(lucene.EmptyTokenStream()).payload is None
+            assert engine.TokenFilter(miscellaneous.EmptyTokenStream()).payload is None
             assert token.term == 'hello'
             assert token.type == token.payload == '<ALPHANUM>'
             assert token.offset == (0, 5)
@@ -53,25 +65,24 @@ class TestCase(BaseTest):
             token.offset, token.positionIncrement = (0, 0), 0
         assert str(stemmer.parse('hellos', field=['body', 'title'])) == 'body:hello title:hello'
         assert str(stemmer.parse('hellos', field={'body': 1.0, 'title': 2.0})) == 'body:hello title:hello^2.0'
-        indexer = engine.Indexer(analyzer=stemmer, version=lucene.Version.LUCENE_30, writeLockTimeout=100L)
-        assert indexer.writeLockTimeout == 100
+        indexer = engine.Indexer(analyzer=stemmer, version=util.Version.LUCENE_30, writeLockTimeout=100L)
+        assert indexer.config.writeLockTimeout == 100
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
-        indexer.set('name', store=True, index=False, boost=2.0)
-        for field in indexer.fields['name'].items('sample'):
-            assert isinstance(field, lucene.Field) and field.boost == 2.0
-        indexer.set('tag', store=True, index=True)
+        indexer.set('name', store=True, index=False)
+        indexer.set('tag', store=True, index=True, boost=2.0)
+        for field in indexer.fields['tag'].items('sample'):
+            assert isinstance(field, document.Field) and getattr(field, 'getBoost', field.boost)() == 2.0
         searcher = indexer.indexSearcher
         indexer.commit()
         assert searcher is indexer.indexSearcher
         assert not searcher.search(count=1)
         indexer.add(text='hello worlds', name='sample', tag=['python', 'search'])
         assert len(indexer) == 1 and list(indexer) == []
-        assert not indexer.optimized
         indexer.commit()
         assert searcher is not indexer.indexSearcher
         assert list(indexer) == [0]
-        assert indexer.current and indexer.optimized
+        assert indexer.current
         assert 0 in indexer and 1 not in indexer
         doc = indexer[0]
         assert doc == {'tag': ['python', 'search'], 'name': ['sample']}
@@ -84,8 +95,16 @@ class TestCase(BaseTest):
         assert not list(indexer.termvector(0, 'tag'))
         assert indexer.count('text', 'hello') == indexer.count('text:hello') == 1
         assert sorted(indexer.names()) == ['name', 'tag', 'text']
-        assert sorted(indexer.names('indexed', isIndexed=True)) == ['tag', 'text']
-        assert indexer.names('unindexed', isIndexed=False) == ['name']
+        try:
+            names = indexer.names(indexed=True)
+        except AttributeError:
+            names = indexer.names('indexed', isIndexed=True)
+        assert sorted(names)[-2:] == ['tag', 'text']
+        try:
+            names = indexer.names(indexed=False)
+        except AttributeError:
+            names = indexer.names('unindexed', isIndexed=False)
+        assert 'name' in names
         assert list(indexer.terms('text')) == ['hello', 'world']
         assert list(indexer.terms('text', 'h', 'v')) == ['hello']
         assert dict(indexer.terms('text', 'w', counts=True)) == {'world': 1}
@@ -97,7 +116,9 @@ class TestCase(BaseTest):
         assert list(indexer.positions('text', 'world', payloads=True)) == [(0, [(1, '<ALPHANUM>')])]
         hits = indexer.search('text:hello')
         assert len(hits) == hits.count == 1
-        assert hits.ids == [0]
+        self.assertRaises(AssertionError, hits.__getitem__, slice(None, None, 2))
+        assert hits.scoredocs is hits[:1].scoredocs and not hits[1:]
+        assert list(hits.ids) == [0]
         score, = hits.scores
         assert 0 < score < 1
         assert dict(hits.items()) == {0: score}
@@ -109,17 +130,23 @@ class TestCase(BaseTest):
         query = engine.Query.multiphrase('text', ('hello', 'hi'), None, 'world')
         assert str(query).startswith('text:"(hello hi) ') and list(query.positions) == [0, 2]
         query = engine.Query.wildcard('text', '*')
-        assert str(query) == 'text:*' and isinstance(query, lucene.WildcardQuery)
-        assert str(lucene.MatchAllDocsQuery() | query) == '*:* text:*'
-        assert str(lucene.MatchAllDocsQuery() - query) == '*:* -text:*'
+        assert str(query) == 'text:*' and isinstance(query, search.WildcardQuery)
+        assert str(search.MatchAllDocsQuery() | query) == '*:* text:*'
+        assert str(search.MatchAllDocsQuery() - query) == '*:* -text:*'
         query = +query
-        query &= engine.Query.fuzzy('text', 'hello')
-        query |= engine.Query.fuzzy('text', 'hello', 0.1)
-        assert str(query) == '+text:* +text:hello~0.5 text:hello~0.1'
+        if hasattr(search.FuzzyQuery, 'defaultMaxEdits'):
+            query &= engine.Query.fuzzy('text', 'hello')
+            query |= engine.Query.fuzzy('text', 'hello', 1)
+            assert str(query) == '+text:* +text:hello~2 text:hello~1'
+        else:
+            query &= engine.Query.fuzzy('text', 'hello')
+            query |= engine.Query.fuzzy('text', 'hello', 0.1)
+            assert str(query) == '+text:* +text:hello~0.5 text:hello~0.1'
         query = engine.Query.span('text', 'world')
         assert str(query.mask('name')) == 'mask(text:world) as name'
         assert str(query.payload()) == 'spanPayCheck(text:world, payloadRef: )'
-        assert isinstance(query.filter(cache=False), lucene.SpanQueryFilter) and isinstance(query.filter(), lucene.CachingSpanFilter)
+        assert isinstance(query.filter(cache=False), getattr(search, 'SpanQueryFilter', search.QueryWrapperFilter))
+        assert isinstance(query.filter(), getattr(search, 'CachingSpanFilter', search.CachingWrapperFilter))
         query = engine.Query.disjunct(0.1, query, name='sample')
         assert str(query) == '(text:world | name:sample)~0.1'
         query = engine.Query.near('text', 'hello', ('tag', 'python'), slop=-1, inOrder=False)
@@ -140,7 +167,11 @@ class TestCase(BaseTest):
         indexer.commit()
         assert 0 not in indexer and len(indexer) == 0 and sum(indexer.segments.values()) == 0
         indexer.add(tag='test', name='old')
-        indexer.update('tag', boost=2.0, tag='test')
+        if hasattr(document.Document, 'boost'):
+            with assertWarns(DeprecationWarning):
+                indexer.update('tag', boost=2.0, tag='test')
+        else:
+            indexer.update('tag', tag='test')
         indexer.commit()
         assert [indexer[id].dict() for id in indexer] == [{'tag': 'test'}]
         indexer.update('tag', 'test', {'name': 'new'})
@@ -155,21 +186,18 @@ class TestCase(BaseTest):
         indexer += temp.directory
         indexer += self.tempdir
         assert len(indexer) == 3
-        indexer.add(text=lucene.WhitespaceTokenizer(lucene.StringReader('?')), name=lucene.JArray_byte('{}'))
+        indexer.add(text=analysis.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, StringReader('?')), name=lucene.JArray_byte('{}'))
         indexer.commit()
-        assert indexer[next(indexer.docs('text', '?'))] == {'name': ['{}']}
+        value = indexer[next(indexer.docs('text', '?'))]['name']
+        assert value == '{}' or value.utf8ToString() == '{}'
         reader = engine.indexers.IndexReader(indexer.indexReader)
         assert reader[0].dict() == {} and reader.count('text', '?') == 1
         assert len(reader.comparator('text')) == 4
         indexer.delete('text', '?')
-        with assertWarns(DeprecationWarning):
-            indexer.commit(expunge=True)
         indexer.commit(merge=True)
         assert not indexer.hasDeletions()
-        indexer.commit(merge=2)
-        with assertWarns(DeprecationWarning):
-            indexer.commit(optimize=True)
-        assert indexer.optimized
+        indexer.commit(merge=1)
+        assert len(list(indexer.readers)) == 1
         del reader.indexReader
         self.assertRaises(AttributeError, getattr, reader, 'maxDoc')
         del indexer.indexSearcher
@@ -182,13 +210,13 @@ class TestCase(BaseTest):
         for name, params in fixture.constitution.fields.items():
             indexer.set(name, **params)
         for doc in fixture.constitution.docs():
-            indexer.add(doc, boost=('article' in doc) + 1.0)
+            indexer.add(doc)
         indexer.commit()
         searcher = engine.IndexSearcher.load(self.tempdir)
         engine.IndexSearcher.load(searcher.directory) # ensure directory isn't closed
-        assert len(indexer) == len(searcher) and lucene.RAMDirectory.instance_(searcher.directory)
+        assert len(indexer) == len(searcher) and store.RAMDirectory.instance_(searcher.directory)
         assert indexer.filters == indexer.spellcheckers == {}
-        assert indexer.facets(lucene.MatchAllDocsQuery(), 'amendment')
+        assert indexer.facets(search.MatchAllDocsQuery(), 'amendment')
         assert indexer.suggest('amendment', '')
         assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
         indexer.delete('amendment', doc['amendment'])
@@ -197,21 +225,22 @@ class TestCase(BaseTest):
         indexer.commit(filters=True, spellcheckers=True)
         assert reader.refCount == 0
         assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
-        doc['amendment'] = engine.Analyzer(lucene.WhitespaceTokenizer).tokens(doc['amendment'])
-        doc['date'] = engine.Analyzer(lucene.WhitespaceTokenizer).tokens(doc['date']), 2.0
+        tokenizer = lambda reader: analysis.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, reader)
+        doc['amendment'] = engine.Analyzer(tokenizer).tokens(doc['amendment'])
+        doc['date'] = engine.Analyzer(tokenizer).tokens(doc['date']), 2.0
         scores = list(searcher.match(doc, 'text:congress', 'text:law', 'amendment:27', 'date:19*'))
         assert 0.0 == scores[0] < scores[1] < scores[2] < scores[3] == 1.0
         searcher = engine.MultiSearcher([indexer.indexReader, self.tempdir])
-        assert searcher.refCount == 1
+        assert searcher.refCount == 1 and searcher.timestamp
         assert searcher.count() == len(searcher) == 2 * len(indexer)
         searcher.sorters['amendment'] = engine.SortField('amenmdment', int)
         comparator = searcher.comparator('amendment')
-        assert comparator is searcher.comparator('amendment') and set(map(type, comparator)) == set([int])
+        assert set(map(type, comparator)) == set([int])
         assert searcher is searcher.reopen()
-        assert searcher.facets(lucene.MatchAllDocsQuery(), 'amendment')['amendment'] == dict.fromkeys(map(str, range(1, 28)), 2)
+        assert searcher.facets(search.MatchAllDocsQuery(), 'amendment')['amendment'] == dict.fromkeys(map(str, range(1, 28)), 2)
         reader = searcher.indexReader
         del searcher
-        self.assertRaises(lucene.JavaError, reader.isCurrent)
+        assert not reader.refCount
         assert len(indexer) == len(indexer.search()) == 35
         assert sorted(indexer.names()) == ['amendment', 'article', 'date', 'text']
         articles = list(indexer.terms('article'))
@@ -220,6 +249,9 @@ class TestCase(BaseTest):
         assert sorted(map(int, indexer.terms('amendment'))) == range(1, 28)
         assert list(itertools.islice(indexer.terms('text', 'right'), 2)) == ['right', 'rights']
         assert list(indexer.terms('text', 'right*')) == ['right', 'rights']
+        if hasattr(search, 'WildcardTermEnum'):
+            with assertWarns(DeprecationWarning):
+                assert list(indexer.terms('text', 'right?')) == ['rights']
         assert list(indexer.terms('text', 'right', minSimilarity=0.5)) == ['eight', 'right', 'rights']
         word, count = next(indexer.terms('text', 'people', counts=True))
         assert word == 'people' and count == 8
@@ -232,33 +264,36 @@ class TestCase(BaseTest):
         assert hit['article'] == 'Preamble'
         assert sorted(hit.dict()) == ['__id__', '__score__', 'article']
         hits = indexer.search('people', field='text')
-        assert hits[0]['article'] == 'Preamble'
+        assert 'Preamble' in (hit.get('article') for hit in hits)
         assert len(hits) == hits.count == 8
         assert set(map(type, hits.ids)) == set([int]) and set(map(type, hits.scores)) == set([float])
         assert hits.maxscore == max(hits.scores)
-        ids = hits.ids
+        ids = list(hits.ids)
         hits = indexer.search('people', count=5, field='text')
-        assert hits.ids == ids[:len(hits)]
+        assert list(hits.ids) == ids[:len(hits)]
         assert len(hits) == 5 and hits.count == 8
         assert not any(map(math.isnan, hits.scores))
         assert hits.maxscore == max(hits.scores)
-        hits = indexer.search('text:people', count=5, sort=lucene.Sort.INDEXORDER)
-        assert sorted(hits.ids) == hits.ids
+        hits = indexer.search('text:people', count=5, sort=search.Sort.INDEXORDER)
+        assert sorted(hits.ids) == list(hits.ids)
         sort = engine.SortField('amendment', type=int)
         hits = indexer.search('text:people', count=5, sort=sort)
         assert [hit.get('amendment') for hit in hits] == [None, None, '1', '2', '4']
+        assert [key for hit in hits for key in hit.keys]== [0, 0, 1, 2, 4]
         assert all(map(math.isnan, hits.scores))
         hits = indexer.search('text:right', count=10**7, sort=sort, scores=True)
         assert not any(map(math.isnan, hits.scores)) and sorted(hits.scores, reverse=True) != hits.scores
         assert math.isnan(hits.maxscore)
         hits = indexer.search('text:right', count=2, sort=sort, maxscore=True)
         assert hits.maxscore > max(hits.scores)
-        comparator = indexer.comparator('amendment', type=int, parser=lambda value: int(value or -1))
-        hits = indexer.search('text:people', sort=comparator.__getitem__)
-        assert sorted(hits.ids) == hits.ids and hits.ids != ids
+        parser = lambda value: int((value.utf8ToString() if hasattr(value, 'utf8ToString') else value) or -1)
+        comparator = indexer.comparator('amendment', type=int, parser=parser)
+        with assertWarns(DeprecationWarning):
+            hits = indexer.search('text:people', sort=comparator.__getitem__)
+        assert sorted(hits.ids) == list(hits.ids) and list(hits.ids) != ids
         comparator = list(zip(*map(indexer.comparator, ['article', 'amendment'])))
         hits = indexer.search('text:people', sort=comparator.__getitem__)
-        assert sorted(hits.ids) != hits.ids
+        assert sorted(hits.ids) != list(hits.ids)
         hits = indexer.search('text:people', count=5, sort='amendment', reverse=True)
         assert [hit['amendment'] for hit in hits] == ['9', '4', '2', '17', '10']
         hit, = indexer.search('freedom', field='text')
@@ -270,7 +305,6 @@ class TestCase(BaseTest):
         query = engine.Query.term('text', 'right', boost=2.0)
         assert query.boost == 2.0
         assert indexer.facets(str(query), 'amendment', 'article') == {'amendment': 12, 'article': 1}
-        self.assertRaises(TypeError, indexer.overlap, query.filter(), lucene.QueryWrapperFilter(query))
         hits = indexer.search('text:people', filter=query.filter())
         assert len(hits) == 4
         hit, = indexer.search('date:192*')
@@ -297,7 +331,7 @@ class TestCase(BaseTest):
         assert str(-query) == '-text:persons'
         query = +query
         query -= engine.Query.term('text', 'papers')
-        assert set(query.terms()) == set([('text', 'persons'), ('text', 'papers')])
+        assert ('text', 'persons') in query.terms()
         assert str(query[-1]) == '-text:papers'
         assert len(query) == len(list(query)) == 2
         span = engine.Query.span('text', 'persons')
@@ -318,15 +352,21 @@ class TestCase(BaseTest):
         assert dict(indexer.termvector(id, 'text', counts=True))['persons'] == 2
         assert dict(indexer.positionvector(id, 'text'))['persons'] in ([3, 26], [10, 48])
         assert dict(indexer.positionvector(id, 'text', offsets=True))['persons'] == [(46, 53), (301, 308)]
-        query = indexer.morelikethis(0)
+        analyzer = analysis.WhitespaceAnalyzer(util.Version.LUCENE_CURRENT)
+        query = indexer.morelikethis(0, analyzer=analyzer)
         assert str(query) == 'text:united text:states'
         hits = indexer.search(query & engine.Query.prefix('article', ''))
         assert len(hits) == 8 and hits[0]['article'] == 'Preamble'
-        assert str(indexer.morelikethis(0, 'article')) == ''
-        assert str(indexer.morelikethis(0, minDocFreq=3)) == 'text:establish text:united text:states'
-        assert str(indexer.morelikethis('jury', 'text', minDocFreq=4, minTermFreq=1)) == 'text:jury'
-        assert str(indexer.morelikethis('jury', 'article')) == ''
-        self.assertRaises(lucene.JavaError, indexer.morelikethis, 'jury')
+        assert str(indexer.morelikethis(0, 'article', analyzer=analyzer)) == ''
+        assert str(indexer.morelikethis(0, minDocFreq=3, analyzer=analyzer)) == 'text:establish text:united text:states'
+        assert str(indexer.morelikethis('jury', 'text', minDocFreq=4, minTermFreq=1, analyzer=analyzer)) == 'text:jury'
+        assert str(indexer.morelikethis('jury', 'article', analyzer=analyzer)) == ''
+        try:
+            query = indexer.morelikethis('jury')
+        except lucene.JavaError:
+            pass
+        else:
+            assert str(query) == ''
         assert indexer.suggest('missing', '') == list(indexer.correct('missing', '')) == []
         assert indexer.suggest('text', '')[:8] == ['shall', 'states', 'any', 'have', 'united', 'congress', 'state', 'constitution']
         assert indexer.suggest('text', 'con')[:2] == ['congress', 'constitution']
@@ -341,9 +381,9 @@ class TestCase(BaseTest):
         assert list(indexer.correct('text', 'write', distance=1, minSimilarity=0.7)) == ['writs', 'writ']
         assert list(indexer.correct('text', 'write', minSimilarity=0.9)) == ['writs', 'writ', 'crime', 'written']
         query = indexer.parse('text:write', spellcheck=True)
-        assert lucene.TermQuery.instance_(query) and str(query) == 'text:writs'
+        assert search.TermQuery.instance_(query) and str(query) == 'text:writs'
         query = indexer.parse('"hello world"', field='text', spellcheck=True)
-        assert lucene.PhraseQuery.instance_(query) and str(query) == 'text:"held would"'
+        assert search.PhraseQuery.instance_(query) and str(query) == 'text:"held would"'
         assert str(indexer.parse('vwxyz', field='text', spellcheck=True)) == 'text:vwxyz'
         with indexer.snapshot() as commit:
             self.assertRaises(lucene.JavaError, indexer.snapshot().__enter__)
@@ -374,8 +414,16 @@ class TestCase(BaseTest):
                 location = '.'.join(doc[name] for name in ['state', 'county', 'city'])
                 indexer.add(doc, latitude=lat, longitude=lng, location=location)
         indexer.commit()
-        assert set(['state', 'zipcode']) < set(indexer.names('indexed', isIndexed=True))
-        assert set(['latitude', 'longitude', 'county', 'city']) == set(indexer.names('unindexed', isIndexed=False))
+        try:
+            names = indexer.names(indexed=True)
+        except AttributeError:
+            names = indexer.names('indexed', isIndexed=True)
+        assert set(['state', 'zipcode']) < set(names)
+        try:
+            names = indexer.names(indexed=False)
+        except AttributeError:
+            names = indexer.names('unindexed', isIndexed=False)
+        assert set(['latitude', 'longitude', 'county', 'city']) <= set(names)
         states = list(indexer.terms('state'))
         assert states[0] == 'AK' and states[-1] == 'WY'
         counties = [term.split('.')[-1] for term in indexer.terms('state.county', 'CA', 'CA~')]
@@ -396,24 +444,39 @@ class TestCase(BaseTest):
         assert la == 'CA.Los Angeles' and facets[la] > 100
         assert orange == 'CA.Orange' and facets[orange] > 10
         (field, facets), = indexer.facets(query, ('state.county', 'CA.*')).items()
-        assert all(value.startswith('CA.') for value in facets) and set(facets) < set(indexer.filters['state.county'])
+        assert all(value.startswith('CA.') for value in facets) and set(facets) < set(indexer.filters[field])
+        if lucene.VERSION >= '3.3':
+            assert set(indexer.grouping('state', count=1)) < set(indexer.grouping('state')) == set(states)
+            grouper = indexer.grouping(field, query, sort=search.Sort(indexer.sorter(field)))
+            assert len(grouper) == 2 and list(grouper) == [la, orange]
+            for value, (name, count) in zip(grouper, grouper.facets(None)):
+                assert value == name and count > 0
+            grouper = indexer.groupings[field] = indexer.grouping(field, engine.Query.term('state', 'CA'))
+            assert indexer.facets(query, field)[field] == facets
+            hits = next(grouper.groups())
+            assert hits.value == 'CA.Los Angeles' and hits.count > 100 and len(hits) == 1
+            hit, = hits
+            assert hit.score in hit.keys
+            assert hit['county'] == 'Los Angeles' and hits.maxscore >= hit.score > 0
+            hits = next(grouper.groups(count=2, sort=search.Sort(indexer.sorter('zipcode')), scores=True))
+            assert hits.value == 'CA.Los Angeles' and math.isnan(hits.maxscore) and len(hits) == 2
+            assert all(hit.score > 0 and hit['zipcode'] > '90000' and hit['zipcode'] in hit.keys for hit in hits)
         for count in (None, len(indexer)):
             hits = indexer.search(query, count=count, timeout=0.01)
             assert 0 <= len(hits) <= indexer.count(query) and hits.count in (None, len(hits)) and hits.maxscore in (None, 1.0)
             hits = indexer.search(query, count=count, timeout=-1)
             assert len(hits) == 0 and hits.count is hits.maxscore is None
         self.assertRaises(lucene.JavaError, indexer.search, filter=Filter())
-        directory = lucene.RAMDirectory()
+        directory = store.RAMDirectory()
         query = engine.Query.term('state', 'CA')
         size = indexer.copy(directory, query)
         searcher = engine.IndexSearcher(directory)
         assert len(searcher) == size and list(searcher.terms('state')) == ['CA']
         path = os.path.join(self.tempdir, 'temp')
-        with assertWarns(DeprecationWarning):
-            size = indexer.copy(path, exclude=query, optimize=True)
+        size = indexer.copy(path, exclude=query, merge=1)
         assert len(searcher) + size == len(indexer)
         searcher = engine.IndexSearcher(path)
-        assert searcher.optimized and 'CA' not in searcher.terms('state')
+        assert len(searcher.segments) == 1 and 'CA' not in searcher.terms('state')
         directory.close()
     
     def testSpatial(self):
@@ -451,7 +514,7 @@ class TestCase(BaseTest):
         query = field.within(x, y, 10**4)
         assert len(query) < 3
         distances = indexer.distances(x, y, 'longitude', 'latitude')
-        hits = indexer.search(query, sort=distances.__getitem__)
+        hits = indexer.search(query).sorted(distances.__getitem__)
         assert hits[0]['zipcode'] == zipcode and distances[hits[0].id] < 10
         cities = set(hit['city'] for hit in hits)
         assert city in cities and 100 > len(cities) > 50
@@ -467,15 +530,8 @@ class TestCase(BaseTest):
         hits = hits.filter(lambda id: distances[id] < 10**4)
         assert 0 < len(hits) < sum(counts.values())
         hits = hits.sorted(distances.__getitem__, reverse=True)
-        assert 0 == distances[hits.ids[-1]] < distances[hits.ids[0]] < 10**4
-        if hasattr(lucene, 'LatLongDistanceFilter'):
-            with assertWarns(DeprecationWarning):
-                f = field.filter(x, y, 10**4, 'longitude', 'latitude')
-            ids = indexer.search(query, sort=distances.__getitem__).ids
-            hits = indexer.search(count=10, filter=f, sort=f.sorter())
-            assert len(hits) < len(ids) and hits.ids == ids[:len(hits)]
-            assert all(f[id] <= distances[id] < 10**4 for id in hits.ids)
-            self.assertRaises(KeyError, f.__getitem__, -1)
+        ids = list(hits.ids)
+        assert 0 == distances[ids[-1]] < distances[ids[0]] < 10**4
     
     def testFields(self):
         "Custom fields."
@@ -485,10 +541,15 @@ class TestCase(BaseTest):
         field = engine.Field('', index=True, analyzed=True, omitNorms=True, termvector=True, withPositions=True, withOffsets=True)
         field, = field.items(' ')
         attrs = 'indexed', 'tokenized', 'termVectorStored', 'storePositionWithTermVector', 'storeOffsetWithTermVector', 'omitNorms'
-        assert all(getattr(field, attr) for attr in attrs)
+        try:
+            assert all(getattr(field, attr) for attr in attrs)
+        except AttributeError:
+            attrs = 'indexed', 'tokenized', 'storeTermVectors', 'storeTermVectorPositions', 'storeTermVectorOffsets', 'omitNorms'
+            assert all(getattr(field.fieldType(), attr)() for attr in attrs)
         indexer = engine.Indexer(self.tempdir)
-        indexer.set('amendment', engine.FormatField, format='{0:02d}', store=True)
-        indexer.set('size', engine.FormatField, format='{0:04d}', store=True)
+        with assertWarns(DeprecationWarning):
+            indexer.set('amendment', engine.FormatField, format='{0:02d}', store=True)
+        indexer.set('size', engine.MapField, func='{0:04d}'.format, store=True)
         field = indexer.fields['date'] = engine.NestedField('Y-m-d', sep='-', store=True)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
@@ -512,26 +573,30 @@ class TestCase(BaseTest):
         sizes = dict((id, int(indexer[id]['size'])) for id in indexer)
         ids = sorted((id for id in sizes if sizes[id] >= 1000), key=sizes.get)
         query = engine.Query.range('size', '1000', None)
-        hits = indexer.search(query, sort=sizes.get)
-        assert hits.ids == ids
+        hits = indexer.search(query).sorted(sizes.get)
+        assert list(hits.ids) == ids
         hits = indexer.search(query, count=3, sort=engine.SortField('size', type=long))
-        assert hits.ids == ids[:len(hits)]
+        assert list(hits.ids) == ids[:len(hits)]
         query = engine.Query.range('size', None, '1000')
         assert indexer.count(query) == len(sizes) - len(ids)
-        indexer.sorters['year'] = engine.SortField('Y-m-d', type=int, parser=lambda date: int(date.split('-')[0]))
-        assert indexer.comparator('year')[:10] == [1791] * 10
-        cache = len(lucene.FieldCache.DEFAULT.cacheEntries)
+        parser = lambda date: int((date.utf8ToString() if lucene.VERSION >= '4' else date).split('-')[0])
+        indexer.sorters['year'] = engine.SortField('Y-m-d', type=int, parser=parser)
+        assert list(indexer.comparator('year'))[:10] == [1791] * 10
+        cache = len(search.FieldCache.DEFAULT.cacheEntries)
         hits = indexer.search(count=3, sort='year')
         assert [int(hit['amendment']) for hit in hits] == [1, 2, 3]
         hits = indexer.search(count=3, sort='year', reverse=True)
         assert [int(hit['amendment']) for hit in hits] == [27, 26, 25]
-        assert indexer.count(filter=indexer.sorters['year'].filter(None, 1792)) == 10
-        assert cache == len(lucene.FieldCache.DEFAULT.cacheEntries)
+        filter = indexer.sorters['year'].filter(None, 1792)
+        assert indexer.count(filter=filter) == 10
+        assert set(indexer.sorters['year'].terms(filter, *indexer.readers)) == set([1791])
+        assert cache == len(search.FieldCache.DEFAULT.cacheEntries)
         indexer.add()
         indexer.commit(sorters=True)
-        cache = len(lucene.FieldCache.DEFAULT.cacheEntries)
-        assert indexer.comparator('year')[-1] == 0
-        assert cache == len(lucene.FieldCache.DEFAULT.cacheEntries)
+        cache = len(search.FieldCache.DEFAULT.cacheEntries)
+        assert list(indexer.comparator('year'))[-1] == 0
+        assert cache == len(search.FieldCache.DEFAULT.cacheEntries)
+        self.assertRaises(AttributeError, indexer.comparator, 'size', type='score')
     
     def testNumeric(self):
         "Numeric fields."
@@ -563,15 +628,15 @@ class TestCase(BaseTest):
         sizes = dict((id, int(indexer[id]['size'])) for id in indexer)
         ids = sorted((id for id in sizes if sizes[id] >= 1000), key=sizes.get)
         query = field.range(1000, None)
-        hits = indexer.search(query, sort=sizes.get)
-        assert hits.ids == ids
+        hits = indexer.search(query).sorted(sizes.get)
+        assert list(hits.ids) == ids
         hits = indexer.search(query, count=3, sort=engine.SortField('size', type=long))
-        assert hits.ids == ids[:len(hits)]
+        assert list(hits.ids) == ids[:len(hits)]
         query = field.range(None, 1000)
         assert indexer.count(query) == len(sizes) - len(ids)
         self.assertRaises(OverflowError, list, field.items(-2**64))
         nf, = field.items(0.5)
-        assert nf.numericValue.doubleValue() == 0.5
+        assert (nf.numericValue if hasattr(document, 'NumericField') else nf.numericValue()).doubleValue() == 0.5
         assert str(field.range(-2**64, 0)) == 'size:[* TO 0}'
         assert str(field.range(0, 2**64)) == 'size:[0 TO *}'
         assert str(field.range(0.5, None, upper=True)) == 'size:[0.5 TO *]'
@@ -599,24 +664,24 @@ class TestCase(BaseTest):
         id = 3
         text = indexer[id]['text']
         query = '"persons, houses, papers"'
-        highlighter = indexer.highlighter(query, '', terms=True, fields=True, formatter=lucene.SimpleHTMLFormatter('*', '*'))
+        highlighter = indexer.highlighter(query, '', terms=True, fields=True, formatter=highlight.SimpleHTMLFormatter('*', '*'))
         fragments = highlighter.fragments(text, count=3)
         assert len(fragments) == 2 and fragments[0].count('*') == 2*3 and '*persons*' in fragments[1]
         highlighter = indexer.highlighter(query, '', terms=True)
-        highlighter.textFragmenter = lucene.SimpleFragmenter(200)
+        highlighter.textFragmenter = highlight.SimpleFragmenter(200)
         fragment, = highlighter.fragments(text, count=3)
         assert len(fragment) > len(text) and fragment.count('<B>persons</B>') == 2
         fragment, = indexer.highlighter(query, 'text', tag='em').fragments(id, count=3)
         assert len(fragment) < len(text) and fragment.index('<em>persons') < fragment.index('papers</em>')
         fragment, = indexer.highlighter(query, 'text').fragments(id)
         assert fragment.count('<b>') == fragment.count('</b>') == 1
-        highlighter = indexer.highlighter(query, 'text', fragListBuilder=lucene.SingleFragListBuilder())
+        highlighter = indexer.highlighter(query, 'text', fragListBuilder=vectorhighlight.SingleFragListBuilder())
         text, = highlighter.fragments(id)
         assert fragment in text and len(text) > len(fragment)
     
     def testNearRealTime(self):
         "Near real-time index updates."
-        indexer = engine.Indexer(version=lucene.Version.LUCENE_30, nrt=True)
+        indexer = engine.Indexer(version=util.Version.LUCENE_30, nrt=True)
         indexer.add()
         assert indexer.count() == 0 and not indexer.current
         indexer.refresh(filters=True)
@@ -629,7 +694,7 @@ class TestCase(BaseTest):
     
     def testFilters(self):
         "Custom filters."
-        if not hasattr(lucene, 'FixedBitSet'):
+        if lucene.VERSION < '3.5':
             return self.assertRaises(AssertionError, engine.queries.TermsFilter, '')
         indexer = engine.Indexer()
         indexer.set('name', store=True, index=True)
@@ -637,7 +702,7 @@ class TestCase(BaseTest):
             indexer.add(name=name)
         indexer.commit()
         filter = engine.TermsFilter('name')
-        assert indexer.count(filter=filter) == len(filter.readers) == 0
+        assert len(filter.readers) == 0
         filter.add('alpha', 'bravo')
         filter.discard('bravo', 'charlie')
         assert filter.values == set(['alpha'])
@@ -651,7 +716,7 @@ class TestCase(BaseTest):
         indexer.add(name='delta')
         indexer.delete('name', 'alpha')
         indexer.commit()
-        assert filter.readers > set(indexer.sequentialSubReaders)
+        assert filter.readers > set(indexer.readers)
         assert [hit['name'] for hit in indexer.search(filter=filter)] == ['bravo', 'delta']
         parallel.update('bravo')
         parallel.update('charlie', priority='high')
@@ -659,7 +724,7 @@ class TestCase(BaseTest):
         assert [hit['name'] for hit in indexer.search(filter=filter)] == ['charlie', 'delta']
         parallel.commit()
         filter.refresh(indexer)
-        assert filter.readers == set(indexer.sequentialSubReaders)
+        assert filter.readers == set(indexer.readers)
 
 if __name__ == '__main__':
     lucene.initVM()
